/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_CURVEELEMENTSTRUCT_H
#define DPC_IMPL_TYPE_CURVEELEMENTSTRUCT_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"

namespace dpc {
struct CurveElementStruct {
    ::Float longitudinal_distance_start{ 0.0F };
    ::Float longitudinal_distance_end{ 0.0F };
    ::Float c0{ 0.0F };
    ::Float c1{ 0.0F };
    ::Float c2{ 0.0F };
    ::Float c3{ 0.0F };
    ::Float std_of_c0{ 0.0F };
    ::Float std_of_c1{ 0.0F };
    ::Float std_of_c2{ 0.0F };
    ::Float std_of_c3{ 0.0F };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(longitudinal_distance_start);
        fun(longitudinal_distance_end);
        fun(c0);
        fun(c1);
        fun(c2);
        fun(c3);
        fun(std_of_c0);
        fun(std_of_c1);
        fun(std_of_c2);
        fun(std_of_c3);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(longitudinal_distance_start);
        fun(longitudinal_distance_end);
        fun(c0);
        fun(c1);
        fun(c2);
        fun(c3);
        fun(std_of_c0);
        fun(std_of_c1);
        fun(std_of_c2);
        fun(std_of_c3);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("longitudinal_distance_start", longitudinal_distance_start);
        fun("longitudinal_distance_end", longitudinal_distance_end);
        fun("c0", c0);
        fun("c1", c1);
        fun("c2", c2);
        fun("c3", c3);
        fun("std_of_c0", std_of_c0);
        fun("std_of_c1", std_of_c1);
        fun("std_of_c2", std_of_c2);
        fun("std_of_c3", std_of_c3);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("longitudinal_distance_start", longitudinal_distance_start);
        fun("longitudinal_distance_end", longitudinal_distance_end);
        fun("c0", c0);
        fun("c1", c1);
        fun("c2", c2);
        fun("c3", c3);
        fun("std_of_c0", std_of_c0);
        fun("std_of_c1", std_of_c1);
        fun("std_of_c2", std_of_c2);
        fun("std_of_c3", std_of_c3);
    }

    friend bool operator==(const ::dpc::CurveElementStruct& lhs, const ::dpc::CurveElementStruct& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.longitudinal_distance_start - rhs.longitudinal_distance_start)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.longitudinal_distance_end - rhs.longitudinal_distance_end)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.c0 - rhs.c0)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.c1 - rhs.c1)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.c2 - rhs.c2)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.c3 - rhs.c3)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.std_of_c0 - rhs.std_of_c0)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.std_of_c1 - rhs.std_of_c1)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.std_of_c2 - rhs.std_of_c2)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.std_of_c3 - rhs.std_of_c3)) < DBL_EPSILON);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_CURVEELEMENTSTRUCT_H
